@@ -34,6 +34,7 @@
 #include <cairo-gobject.h>
 
 #include <eel/eel-glib-extensions.h>
+#include <eel/eel-stock-dialogs.h>
 
 #include "caja-icon-info.h"
 #include "caja-global-preferences.h"
@@ -135,7 +136,10 @@ add_elem_to_str_array (char **v, const char *s)
 
     len = v != NULL ? g_strv_length (v) : 0;
     r = g_new0 (char *, len + 2);
-    memcpy (r, v, len * sizeof (char *));
+
+    if (v)
+        memcpy (r, v, len * sizeof (char *));
+
     r[len] = g_strdup (s);
     r[len+1] = NULL;
     g_free (v);
@@ -752,15 +756,17 @@ static gboolean
 is_shift_pressed (void)
 {
     gboolean ret;
+    GdkDisplay *display;
     XkbStateRec state;
     Bool status;
 
     ret = FALSE;
 
-    gdk_error_trap_push ();
-    status = XkbGetState (GDK_DISPLAY_XDISPLAY (gdk_display_get_default ()),
-    			  XkbUseCoreKbd, &state);
-    gdk_error_trap_pop_ignored ();
+    display = gdk_display_get_default ();
+    gdk_x11_display_error_trap_push (display);
+    status = XkbGetState (GDK_DISPLAY_XDISPLAY (display),
+                          XkbUseCoreKbd, &state);
+    gdk_x11_display_error_trap_pop_ignored (display);
 
     if (status == Success)
     {
@@ -940,6 +946,7 @@ do_autorun_for_content_type (GMount *mount, const char *x_content_type, CajaAuto
     GtkWidget *always_check_button;
     GtkWidget *eject_button;
     GtkWidget *image;
+    GtkWidget *action_area;
     char *markup;
     char *content_description;
     char *mount_name;
@@ -1142,10 +1149,16 @@ show_dialog:
                       data);
     gtk_box_pack_start (GTK_BOX (vbox), always_check_button, TRUE, TRUE, 0);
 
-    gtk_dialog_add_buttons (GTK_DIALOG (dialog),
-                            GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
-                            GTK_STOCK_OK, GTK_RESPONSE_OK,
-                            NULL);
+    eel_dialog_add_button (GTK_DIALOG (dialog),
+                           _("_Cancel"),
+                           "process-stop",
+                           GTK_RESPONSE_CANCEL);
+
+    action_area = gtk_widget_get_parent (eel_dialog_add_button (GTK_DIALOG (dialog),
+                                                                _("_OK"),
+                                                                "gtk-ok",
+                                                                GTK_RESPONSE_OK));
+
     gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 
     if (g_mount_can_eject (mount))
@@ -1170,7 +1183,7 @@ show_dialog:
         data->should_eject = FALSE;
     }
     gtk_dialog_add_action_widget (GTK_DIALOG (dialog), eject_button, AUTORUN_DIALOG_RESPONSE_EJECT);
-    gtk_button_box_set_child_secondary (GTK_BUTTON_BOX (gtk_dialog_get_action_area (GTK_DIALOG (dialog))), eject_button, TRUE);
+    gtk_button_box_set_child_secondary (GTK_BUTTON_BOX (action_area), eject_button, TRUE);
 
     /* show the dialog */
     gtk_widget_show_all (dialog);
