@@ -37,7 +37,7 @@
 #include "caja-file.h"
 #include "caja-icon-info.h"
 
-struct _CajaFileConflictDialogDetails
+struct _CajaFileConflictDialogPrivate
 {
     /* conflicting objects */
     CajaFile *source;
@@ -63,37 +63,33 @@ struct _CajaFileConflictDialogDetails
     GtkWidget *src_image;
 };
 
-G_DEFINE_TYPE (CajaFileConflictDialog,
-               caja_file_conflict_dialog,
-               GTK_TYPE_DIALOG);
-
-#define CAJA_FILE_CONFLICT_DIALOG_GET_PRIVATE(object)		\
-	(G_TYPE_INSTANCE_GET_PRIVATE ((object), CAJA_TYPE_FILE_CONFLICT_DIALOG, \
-				      CajaFileConflictDialogDetails))
+G_DEFINE_TYPE_WITH_PRIVATE (CajaFileConflictDialog,
+                            caja_file_conflict_dialog,
+                            GTK_TYPE_DIALOG);
 
 static void
 file_icons_changed (CajaFile *file,
                     CajaFileConflictDialog *fcd)
 {
-    GdkPixbuf *pixbuf;
+    cairo_surface_t *surface;
 
-    pixbuf = caja_file_get_icon_pixbuf (fcd->details->destination,
-                                        CAJA_ICON_SIZE_LARGE,
-                                        TRUE,
-                                        gtk_widget_get_scale_factor (fcd->details->dest_image),
-                                        CAJA_FILE_ICON_FLAGS_USE_THUMBNAILS);
+    surface = caja_file_get_icon_surface (fcd->details->destination,
+                                          CAJA_ICON_SIZE_LARGE,
+                                          FALSE,
+                                          gtk_widget_get_scale_factor (fcd->details->dest_image),
+                                          CAJA_FILE_ICON_FLAGS_USE_THUMBNAILS);
 
-    gtk_image_set_from_pixbuf (GTK_IMAGE (fcd->details->dest_image), pixbuf);
-    g_object_unref (pixbuf);
+    gtk_image_set_from_surface (GTK_IMAGE (fcd->details->dest_image), surface);
+    cairo_surface_destroy (surface);
 
-    pixbuf = caja_file_get_icon_pixbuf (fcd->details->source,
-                                        CAJA_ICON_SIZE_LARGE,
-                                        TRUE,
-                                        gtk_widget_get_scale_factor (fcd->details->src_image),
-                                        CAJA_FILE_ICON_FLAGS_USE_THUMBNAILS);
+    surface = caja_file_get_icon_surface (fcd->details->source,
+                                          CAJA_ICON_SIZE_LARGE,
+                                          FALSE,
+                                          gtk_widget_get_scale_factor (fcd->details->src_image),
+                                          CAJA_FILE_ICON_FLAGS_USE_THUMBNAILS);
 
-    gtk_image_set_from_pixbuf (GTK_IMAGE (fcd->details->src_image), pixbuf);
-    g_object_unref (pixbuf);
+    gtk_image_set_from_surface (GTK_IMAGE (fcd->details->src_image), surface);
+    cairo_surface_destroy (surface);
 }
 
 static void
@@ -104,13 +100,13 @@ file_list_ready_cb (GList *files,
     CajaFile *src, *dest, *dest_dir;
     time_t src_mtime, dest_mtime;
     gboolean source_is_dir,	dest_is_dir, should_show_type;
-    CajaFileConflictDialogDetails *details;
+    CajaFileConflictDialogPrivate *details;
     char *primary_text, *message, *secondary_text;
     const gchar *message_extra;
     char *dest_name, *dest_dir_name, *edit_name;
     char *label_text;
     char *size, *date, *type = NULL;
-    GdkPixbuf *pixbuf;
+    cairo_surface_t *surface;
     GtkWidget *label;
     GString *str;
     PangoAttrList *attr_list;
@@ -238,27 +234,27 @@ file_list_ready_cb (GList *files,
     g_free (secondary_text);
 
     /* Set up file icons */
-    pixbuf = caja_file_get_icon_pixbuf (dest,
-                                        CAJA_ICON_SIZE_LARGE,
-                                        TRUE,
-                                        gtk_widget_get_scale_factor (fcd->details->titles_vbox),
-                                        CAJA_FILE_ICON_FLAGS_USE_THUMBNAILS);
-    details->dest_image = gtk_image_new_from_pixbuf (pixbuf);
+    surface = caja_file_get_icon_surface (dest,
+                                          CAJA_ICON_SIZE_LARGE,
+                                          TRUE,
+                                          gtk_widget_get_scale_factor (fcd->details->titles_vbox),
+                                          CAJA_FILE_ICON_FLAGS_USE_THUMBNAILS);
+    details->dest_image = gtk_image_new_from_surface (surface);
     gtk_box_pack_start (GTK_BOX (details->first_hbox),
                         details->dest_image, FALSE, FALSE, 0);
     gtk_widget_show (details->dest_image);
-    g_object_unref (pixbuf);
+    cairo_surface_destroy (surface);
 
-    pixbuf = caja_file_get_icon_pixbuf (src,
-                                        CAJA_ICON_SIZE_LARGE,
-                                        TRUE,
-                                        gtk_widget_get_scale_factor (fcd->details->titles_vbox),
-                                        CAJA_FILE_ICON_FLAGS_USE_THUMBNAILS);
-    details->src_image = gtk_image_new_from_pixbuf (pixbuf);
+    surface = caja_file_get_icon_surface (src,
+                                          CAJA_ICON_SIZE_LARGE,
+                                          TRUE,
+                                          gtk_widget_get_scale_factor (fcd->details->titles_vbox),
+                                          CAJA_FILE_ICON_FLAGS_USE_THUMBNAILS);
+    details->src_image = gtk_image_new_from_surface (surface);
     gtk_box_pack_start (GTK_BOX (details->second_hbox),
                         details->src_image, FALSE, FALSE, 0);
     gtk_widget_show (details->src_image);
-    g_object_unref (pixbuf);
+    cairo_surface_destroy (surface);
 
     /* Set up labels */
     label = gtk_label_new (NULL);
@@ -384,7 +380,7 @@ static void
 build_dialog_appearance (CajaFileConflictDialog *fcd)
 {
     GList *files = NULL;
-    CajaFileConflictDialogDetails *details = fcd->details;
+    CajaFileConflictDialogPrivate *details = fcd->details;
 
     files = g_list_prepend (files, details->source);
     files = g_list_prepend (files, details->destination);
@@ -403,7 +399,7 @@ set_source_and_destination (GtkWidget *w,
                             GFile *dest_dir)
 {
     CajaFileConflictDialog *dialog;
-    CajaFileConflictDialogDetails *details;
+    CajaFileConflictDialogPrivate *details;
 
     dialog = CAJA_FILE_CONFLICT_DIALOG (w);
     details = dialog->details;
@@ -419,7 +415,7 @@ static void
 entry_text_changed_cb (GtkEditable *entry,
                        CajaFileConflictDialog *dialog)
 {
-    CajaFileConflictDialogDetails *details;
+    CajaFileConflictDialogPrivate *details;
 
     details = dialog->details;
 
@@ -453,7 +449,7 @@ static void
 expander_activated_cb (GtkExpander *w,
                        CajaFileConflictDialog *dialog)
 {
-    CajaFileConflictDialogDetails *details;
+    CajaFileConflictDialogPrivate *details;
     int start_pos, end_pos;
 
     details = dialog->details;
@@ -477,7 +473,7 @@ static void
 checkbox_toggled_cb (GtkToggleButton *t,
                      CajaFileConflictDialog *dialog)
 {
-    CajaFileConflictDialogDetails *details;
+    CajaFileConflictDialogPrivate *details;
 
     details = dialog->details;
 
@@ -506,7 +502,7 @@ static void
 reset_button_clicked_cb (GtkButton *w,
                          CajaFileConflictDialog *dialog)
 {
-    CajaFileConflictDialogDetails *details;
+    CajaFileConflictDialogPrivate *details;
     int start_pos, end_pos;
 
     details = dialog->details;
@@ -525,7 +521,7 @@ static void
 diff_button_clicked_cb (GtkButton *w,
                         CajaFileConflictDialog *dialog)
 {
-    CajaFileConflictDialogDetails *details;
+    CajaFileConflictDialogPrivate *details;
     details = dialog->details;
     
     GError *error;
@@ -564,11 +560,11 @@ caja_file_conflict_dialog_init (CajaFileConflictDialog *fcd)
 {
     GtkWidget *hbox, *vbox, *vbox2;
     GtkWidget *widget, *dialog_area;
-    CajaFileConflictDialogDetails *details;
+    CajaFileConflictDialogPrivate *details;
     GtkDialog *dialog;
     gboolean source_is_dir;
 
-    details = fcd->details = CAJA_FILE_CONFLICT_DIALOG_GET_PRIVATE (fcd);
+    details = fcd->details = caja_file_conflict_dialog_get_instance_private (fcd);
     dialog = GTK_DIALOG (fcd);
 
     source_is_dir = caja_file_is_directory (details->source);
@@ -687,7 +683,7 @@ caja_file_conflict_dialog_init (CajaFileConflictDialog *fcd)
 static void
 do_finalize (GObject *self)
 {
-    CajaFileConflictDialogDetails *details =
+    CajaFileConflictDialogPrivate *details =
         CAJA_FILE_CONFLICT_DIALOG (self)->details;
 
     g_free (details->conflict_name);
@@ -720,8 +716,6 @@ static void
 caja_file_conflict_dialog_class_init (CajaFileConflictDialogClass *klass)
 {
     G_OBJECT_CLASS (klass)->finalize = do_finalize;
-
-    g_type_class_add_private (klass, sizeof (CajaFileConflictDialogDetails));
 }
 
 char *
