@@ -30,6 +30,8 @@
 #include <libcaja-private/caja-file-attributes.h>
 #include <libcaja-private/caja-icon-info.h>
 
+G_BEGIN_DECLS
+
 /* CajaFile is an object used to represent a single element of a
  * CajaDirectory. It's lightweight and relies on CajaDirectory
  * to do most of the work.
@@ -144,6 +146,12 @@ void                    caja_file_monitor_add                       (CajaFile
 void                    caja_file_monitor_remove                    (CajaFile                   *file,
         gconstpointer                   client);
 
+/* Synchronously refreshes file info from disk.
+ * This can call caja_file_changed, so don't call this function from any
+ * of the callbacks for that event.
+ */
+void                     caja_file_refresh_info                     (CajaFile                   *file);
+
 /* Waiting for data that's read asynchronously.
  * This interface currently works only for metadata, but could be expanded
  * to other attributes as well.
@@ -429,7 +437,8 @@ int                     caja_file_compare_location                  (CajaFile
 gboolean                caja_file_is_hidden_file                    (CajaFile                   *file);
 gboolean                caja_file_should_show                       (CajaFile                   *file,
         gboolean                        show_hidden,
-        gboolean                        show_foreign);
+        gboolean                        show_foreign,
+        gboolean                        show_backup);
 GList                  *caja_file_list_filter_hidden                (GList                          *files,
         gboolean                        show_hidden);
 
@@ -450,17 +459,17 @@ char *                  caja_file_get_drop_target_uri               (CajaFile
 char *                  caja_file_get_custom_icon                   (CajaFile                   *file);
 
 
-GIcon *                 caja_file_get_gicon                         (CajaFile                   *file,
-        CajaFileIconFlags           flags);
-CajaIconInfo *      caja_file_get_icon                          (CajaFile                   *file,
-        int                             size,
-        int                             scale,
-        CajaFileIconFlags           flags);
-GdkPixbuf *             caja_file_get_icon_pixbuf                   (CajaFile                   *file,
-        int                             size,
-        gboolean                        force_size,
-        int                             scale,
-        CajaFileIconFlags           flags);
+GIcon           *caja_file_get_gicon        (CajaFile         *file,
+                                             CajaFileIconFlags flags);
+CajaIconInfo    *caja_file_get_icon         (CajaFile         *file,
+                                             int               size,
+                                             int               scale,
+                                             CajaFileIconFlags flags);
+cairo_surface_t *caja_file_get_icon_surface (CajaFile         *file,
+                                             int               size,
+                                             gboolean          force_size,
+                                             int               scale,
+                                             CajaFileIconFlags flags);
 
 gboolean                caja_file_has_open_window                   (CajaFile                   *file);
 void                    caja_file_set_has_open_window               (CajaFile                   *file,
@@ -487,12 +496,12 @@ void                    caja_file_list_cancel_call_when_ready       (CajaFileLis
 /* Debugging */
 void                    caja_file_dump                              (CajaFile                   *file);
 
-typedef struct CajaFileDetails CajaFileDetails;
+typedef struct _CajaFilePrivate CajaFilePrivate;
 
 struct CajaFile
 {
     GObject parent_slot;
-    CajaFileDetails *details;
+    CajaFilePrivate *details;
 };
 
 /* This is actually a "protected" type, but it must be here so we can
@@ -589,4 +598,6 @@ typedef struct
     void                 (* poll_for_media)          (CajaFile                   *file);
 } CajaFileClass;
 
+G_END_DECLS
+
 #endif /* CAJA_FILE_H */
