@@ -40,6 +40,8 @@ struct _CajaIconInfo
     GdkPoint *attach_points;
     char *display_name;
     char *icon_name;
+
+    gint  orig_scale;
 };
 
 struct _CajaIconInfoClass
@@ -121,7 +123,8 @@ caja_icon_info_class_init (CajaIconInfoClass *icon_info_class)
 }
 
 CajaIconInfo *
-caja_icon_info_new_for_pixbuf (GdkPixbuf *pixbuf)
+caja_icon_info_new_for_pixbuf (GdkPixbuf *pixbuf,
+                               gint       scale)
 {
     CajaIconInfo *icon;
 
@@ -132,11 +135,14 @@ caja_icon_info_new_for_pixbuf (GdkPixbuf *pixbuf)
         icon->pixbuf = g_object_ref (pixbuf);
     }
 
+    icon->orig_scale = scale;
+
     return icon;
 }
 
 static CajaIconInfo *
-caja_icon_info_new_for_icon_info (GtkIconInfo *icon_info)
+caja_icon_info_new_for_icon_info (GtkIconInfo *icon_info,
+                                  gint         scale)
 {
     CajaIconInfo *icon;
     GdkPoint *points;
@@ -171,6 +177,8 @@ caja_icon_info_new_for_icon_info (GtkIconInfo *icon_info)
         icon->icon_name = basename;
     }
 
+    icon->orig_scale = scale;
+
     return icon;
 }
 
@@ -178,14 +186,9 @@ caja_icon_info_new_for_icon_info (GtkIconInfo *icon_info)
 typedef struct
 {
     GIcon *icon;
+    int scale;
     int size;
-} LoadableIconKey;
-
-typedef struct
-{
-    char *filename;
-    int size;
-} ThemedIconKey;
+} IconKey;
 
 static GHashTable *loadable_icon_cache = NULL;
 static GHashTable *themed_icon_cache = NULL;
@@ -280,25 +283,28 @@ caja_icon_info_clear_caches (void)
 }
 
 static guint
-loadable_icon_key_hash (LoadableIconKey *key)
+icon_key_hash (IconKey *key)
 {
     return g_icon_hash (key->icon) ^ key->size;
 }
 
 static gboolean
-loadable_icon_key_equal (const LoadableIconKey *a,
-                         const LoadableIconKey *b)
+icon_key_equal (const IconKey *a,
+                         const IconKey *b)
 {
     return a->size == b->size &&
+           a->scale == b->scale &&
            g_icon_equal (a->icon, b->icon);
 }
 
-static LoadableIconKey *
-loadable_icon_key_new (GIcon *icon, int size)
+static IconKey *
+icon_key_new (GIcon *icon,
+              int scale,
+              int    size)
 {
-    LoadableIconKey *key;
+    IconKey *key;
 
-    key = g_slice_new (LoadableIconKey);
+    key = g_slice_new (IconKey);
     key->icon = g_object_ref (icon);
     key->size = size;
 
@@ -306,69 +312,40 @@ loadable_icon_key_new (GIcon *icon, int size)
 }
 
 static void
-loadable_icon_key_free (LoadableIconKey *key)
+icon_key_free (IconKey *key)
 {
     g_object_unref (key->icon);
-    g_slice_free (LoadableIconKey, key);
-}
-
-static guint
-themed_icon_key_hash (ThemedIconKey *key)
-{
-    return g_str_hash (key->filename) ^ key->size;
-}
-
-static gboolean
-themed_icon_key_equal (const ThemedIconKey *a,
-                       const ThemedIconKey *b)
-{
-    return a->size == b->size &&
-           g_str_equal (a->filename, b->filename);
-}
-
-static ThemedIconKey *
-themed_icon_key_new (const char *filename, int size)
-{
-    ThemedIconKey *key;
-
-    key = g_slice_new (ThemedIconKey);
-    key->filename = g_strdup (filename);
-    key->size = size;
-
-    return key;
-}
-
-static void
-themed_icon_key_free (ThemedIconKey *key)
-{
-    g_free (key->filename);
-    g_slice_free (ThemedIconKey, key);
+    g_slice_free (IconKey, key);
 }
 
 CajaIconInfo *
 caja_icon_info_lookup (GIcon *icon,
-                       int size)
+                       int size,
+                       int scale)
 {
-    CajaIconInfo *icon_info;
-    GdkPixbuf *pixbuf;
+    GtkIconTheme *icon_theme;
+    GtkIconInfo *gtkicon_info;
 
-    if (G_IS_LOADABLE_ICON (icon))
-    {
-        LoadableIconKey lookup_key;
-        LoadableIconKey *key;
+    CajaIconInfo *icon_info;
+
+    icon_theme = gtk_icon_theme_get_default ();
+
+    if (G_IS_LOADABLE_ICON (icon)) {
+        GdkPixbuf *pixbuf;
+        IconKey lookup_key;
+        IconKey *key;
         GInputStream *stream;
 
-        if (loadable_icon_cache == NULL)
-        {
-            loadable_icon_cache =
-                g_hash_table_new_full ((GHashFunc)loadable_icon_key_hash,
-                                       (GEqualFunc)loadable_icon_key_equal,
-                                       (GDestroyNotify) loadable_icon_key_free,
-                                       (GDestroyNotify) g_object_unref);
+        if (loadable_icon_cache == NULL) {
+            loadable_icon_cache = g_hash_table_new_full ((GHashFunc) icon_key_hash,
+                                                         (GEqualFunc) icon_key_equal,
+                                                         (GDestroyNotify) icon_key_free,
+                                                         (GDestroyNotify) g_object_unref);
         }
 
         lookup_key.icon = icon;
-        lookup_key.size = size;
+        lookup_key.scale = scale;
+        lookup_key.size = size * scale;
 
         icon_info = g_hash_table_lookup (loadable_icon_cache, &lookup_key);
         if (icon_info)
@@ -378,122 +355,98 @@ caja_icon_info_lookup (GIcon *icon,
 
         pixbuf = NULL;
         stream = g_loadable_icon_load (G_LOADABLE_ICON (icon),
-                                       size,
+                                       size * scale,
                                        NULL, NULL, NULL);
         if (stream)
         {
             pixbuf = gdk_pixbuf_new_from_stream_at_scale (stream,
-                                                          size, size, TRUE,
+                                                          size * scale, size * scale,
+                                                          TRUE,
                                                           NULL, NULL);
             g_input_stream_close (stream, NULL, NULL);
             g_object_unref (stream);
         }
 
-        icon_info = caja_icon_info_new_for_pixbuf (pixbuf);
+        if (!pixbuf) {
+            gtkicon_info = gtk_icon_theme_lookup_icon_for_scale (icon_theme,
+                                                                 "text-x-generic",
+                                                                 size,
+                                                                 scale,
+                                                                 GTK_ICON_LOOKUP_FORCE_SIZE);
+            pixbuf = gtk_icon_info_load_icon (gtkicon_info, NULL);
+        }
 
-        key = loadable_icon_key_new (icon, size);
+
+        icon_info = caja_icon_info_new_for_pixbuf (pixbuf, scale);
+
+        key = icon_key_new (icon, scale, size);
         g_hash_table_insert (loadable_icon_cache, key, icon_info);
+        g_clear_object (&pixbuf);
 
         return g_object_ref (icon_info);
-    }
-    else if (G_IS_THEMED_ICON (icon))
-    {
-        const char * const *names;
-        ThemedIconKey lookup_key;
-        ThemedIconKey *key;
-        GtkIconTheme *icon_theme;
-        GtkIconInfo *gtkicon_info;
-        const char *filename;
-
-        if (themed_icon_cache == NULL)
-        {
-            themed_icon_cache =
-                g_hash_table_new_full ((GHashFunc)themed_icon_key_hash,
-                                       (GEqualFunc)themed_icon_key_equal,
-                                       (GDestroyNotify) themed_icon_key_free,
-                                       (GDestroyNotify) g_object_unref);
+    }   else  {
+        IconKey lookup_key;
+        IconKey *key;
+        if (themed_icon_cache == NULL) {
+            themed_icon_cache = g_hash_table_new_full ((GHashFunc) icon_key_hash,
+                                                       (GEqualFunc) icon_key_equal,
+                                                       (GDestroyNotify) icon_key_free,
+                                                       (GDestroyNotify) g_object_unref);
         }
-
-        names = g_themed_icon_get_names (G_THEMED_ICON (icon));
-
-        icon_theme = gtk_icon_theme_get_default ();
-        gtkicon_info = gtk_icon_theme_choose_icon (icon_theme, (const char **)names, size, 0);
-
-        if (gtkicon_info == NULL)
-        {
-            return caja_icon_info_new_for_pixbuf (NULL);
-        }
-
-        filename = gtk_icon_info_get_filename (gtkicon_info);
-        if (filename == NULL) {
-            g_object_unref (gtkicon_info);
-            return caja_icon_info_new_for_pixbuf (NULL);
-        }
-
-        lookup_key.filename = (char *)filename;
+        lookup_key.icon = icon;
+        lookup_key.scale = scale;
         lookup_key.size = size;
 
         icon_info = g_hash_table_lookup (themed_icon_cache, &lookup_key);
-        if (icon_info)
-        {
-            g_object_unref (gtkicon_info);
+        if (icon_info) {
             return g_object_ref (icon_info);
         }
 
-        icon_info = caja_icon_info_new_for_icon_info (gtkicon_info);
+        gtkicon_info = NULL;
 
-        key = themed_icon_key_new (filename, size);
-        g_hash_table_insert (themed_icon_cache, key, icon_info);
+        gtkicon_info = gtk_icon_theme_lookup_by_gicon_for_scale (icon_theme,
+                                                                 icon,
+                                                                 size,
+                                                                 scale,
+                                                                 GTK_ICON_LOOKUP_FORCE_SIZE);
 
+        if (!gtkicon_info) {
+            gtkicon_info = gtk_icon_theme_lookup_icon_for_scale (icon_theme,
+                                                                 "text-x-generic",
+                                                                 size,
+                                                                 scale,
+                                                                 GTK_ICON_LOOKUP_FORCE_SIZE);
+        }
+
+        icon_info = caja_icon_info_new_for_icon_info (gtkicon_info, scale);
         g_object_unref (gtkicon_info);
 
+        key = icon_key_new (icon,scale, size);
+        g_hash_table_insert (themed_icon_cache, key, icon_info);
+
         return g_object_ref (icon_info);
     }
-    else
-    {
-        GdkPixbuf *pixbuf;
-        GtkIconInfo *gtk_icon_info;
 
-        gtk_icon_info = gtk_icon_theme_lookup_by_gicon (gtk_icon_theme_get_default (),
-                        icon,
-                        size,
-                        GTK_ICON_LOOKUP_FORCE_SIZE);
-        if (gtk_icon_info != NULL)
-        {
-            pixbuf = gtk_icon_info_load_icon (gtk_icon_info, NULL);
-            g_object_unref (gtk_icon_info);
-        }
-        else
-        {
-            pixbuf = NULL;
-        }
-
-        icon_info = caja_icon_info_new_for_pixbuf (pixbuf);
-
-        if (pixbuf != NULL) {
-                g_object_unref (pixbuf);
-        }
-
-        return icon_info;
-    }
 }
 
 CajaIconInfo *
 caja_icon_info_lookup_from_name (const char *name,
-                                 int size)
+                                 int size,
+                                 int scale)
 {
     GIcon *icon;
     CajaIconInfo *info;
 
     icon = g_themed_icon_new (name);
-    info = caja_icon_info_lookup (icon, size);
+    info = caja_icon_info_lookup (icon, size, scale);
     g_object_unref (icon);
     return info;
 }
 
 CajaIconInfo *
 caja_icon_info_lookup_from_path (const char *path,
-                                 int size)
+                                 int size,
+                                 int scale)
 {
     GFile *icon_file;
     GIcon *icon;
@@ -501,7 +454,7 @@ caja_icon_info_lookup_from_path (const char *path,
 
     icon_file = g_file_new_for_path (path);
     icon = g_file_icon_new (icon_file);
-    info = caja_icon_info_lookup (icon, size);
+    info = caja_icon_info_lookup (icon, size, scale);
     g_object_unref (icon);
     g_object_unref (icon_file);
     return info;
@@ -532,6 +485,18 @@ caja_icon_info_get_pixbuf_nodefault (CajaIconInfo  *icon)
     return res;
 }
 
+cairo_surface_t *
+caja_icon_info_get_surface_nodefault (CajaIconInfo *icon)
+{
+    GdkPixbuf *pixbuf;
+    cairo_surface_t *surface;
+
+    pixbuf = caja_icon_info_get_pixbuf_nodefault (icon);
+    surface = gdk_cairo_surface_create_from_pixbuf (pixbuf, icon->orig_scale, NULL);
+    g_object_unref (pixbuf);
+
+    return surface;
+}
 
 GdkPixbuf *
 caja_icon_info_get_pixbuf (CajaIconInfo *icon)
@@ -555,6 +520,19 @@ caja_icon_info_get_pixbuf (CajaIconInfo *icon)
     return res;
 }
 
+cairo_surface_t *
+caja_icon_info_get_surface (CajaIconInfo *icon)
+{
+    GdkPixbuf *pixbuf;
+    cairo_surface_t *surface;
+
+    pixbuf = caja_icon_info_get_pixbuf (icon);
+    surface = gdk_cairo_surface_create_from_pixbuf (pixbuf, icon->orig_scale, NULL);
+    g_object_unref (pixbuf);
+
+    return surface;
+}
+
 GdkPixbuf *
 caja_icon_info_get_pixbuf_nodefault_at_size (CajaIconInfo  *icon,
         gsize              forced_size)
@@ -568,8 +546,8 @@ caja_icon_info_get_pixbuf_nodefault_at_size (CajaIconInfo  *icon,
     if (pixbuf == NULL)
         return NULL;
 
-    w = gdk_pixbuf_get_width (pixbuf);
-    h = gdk_pixbuf_get_height (pixbuf);
+    w = gdk_pixbuf_get_width (pixbuf) / icon->orig_scale;
+    h = gdk_pixbuf_get_height (pixbuf) / icon->orig_scale;
     s = MAX (w, h);
     if (s == forced_size)
     {
@@ -584,6 +562,22 @@ caja_icon_info_get_pixbuf_nodefault_at_size (CajaIconInfo  *icon,
     return scaled_pixbuf;
 }
 
+cairo_surface_t *
+caja_icon_info_get_surface_nodefault_at_size (CajaIconInfo *icon,
+                                              gsize         forced_size)
+{
+    GdkPixbuf *pixbuf;
+    cairo_surface_t *surface;
+
+    pixbuf = caja_icon_info_get_pixbuf_nodefault_at_size (icon, forced_size);
+    /*catch the case of caja_icon_info_get_pixbuf_nodefault_at_size returning NULL */
+    if (!pixbuf)
+        return NULL;
+    surface = gdk_cairo_surface_create_from_pixbuf (pixbuf, icon->orig_scale, NULL);
+    g_object_unref (pixbuf);
+
+    return surface;
+}
 
 GdkPixbuf *
 caja_icon_info_get_pixbuf_at_size (CajaIconInfo  *icon,
@@ -595,8 +589,11 @@ caja_icon_info_get_pixbuf_at_size (CajaIconInfo  *icon,
 
     pixbuf = caja_icon_info_get_pixbuf (icon);
 
-    w = gdk_pixbuf_get_width (pixbuf);
-    h = gdk_pixbuf_get_height (pixbuf);
+    if (pixbuf == NULL)
+        return NULL;
+
+    w = gdk_pixbuf_get_width (pixbuf) / icon->orig_scale;
+    h = gdk_pixbuf_get_height (pixbuf) / icon->orig_scale;
     s = MAX (w, h);
     if (s == forced_size)
     {
@@ -611,6 +608,20 @@ caja_icon_info_get_pixbuf_at_size (CajaIconInfo  *icon,
     return scaled_pixbuf;
 }
 
+cairo_surface_t *
+caja_icon_info_get_surface_at_size (CajaIconInfo *icon,
+                                    gsize         forced_size)
+{
+    GdkPixbuf *pixbuf;
+    cairo_surface_t *surface;
+
+    pixbuf = caja_icon_info_get_pixbuf_at_size (icon, forced_size);
+    surface = gdk_cairo_surface_create_from_pixbuf (pixbuf, icon->orig_scale, NULL);
+    g_object_unref (pixbuf);
+
+    return surface;
+}
+
 gboolean
 caja_icon_info_get_embedded_rect (CajaIconInfo  *icon,
                                   GdkRectangle      *rectangle)
