@@ -105,8 +105,7 @@ static CajaFreedesktopDBus *fdb_manager = NULL;
 static char *   caja_application_get_session_data (CajaApplication *self);
 void caja_application_quit (CajaApplication *self);
 
-G_DEFINE_TYPE (CajaApplication, caja_application, GTK_TYPE_APPLICATION);
-struct _CajaApplicationPriv {
+struct _CajaApplicationPrivate {
 	GVolumeMonitor *volume_monitor;
     gboolean no_desktop;
     gboolean force_desktop;
@@ -114,10 +113,9 @@ struct _CajaApplicationPriv {
     gchar *geometry;
 };
 
-
+G_DEFINE_TYPE_WITH_PRIVATE (CajaApplication, caja_application, GTK_TYPE_APPLICATION);
 
 GList *
-
 caja_application_get_spatial_window_list (void)
 {
     return caja_application_spatial_window_list;
@@ -221,7 +219,10 @@ caja_empty_callback_to_ensure_read() {
 
 static void
 open_window (CajaApplication *application,
-         GFile *location, GdkScreen *screen, const char *geometry, gboolean browser_window)
+             GFile *location, 
+             GdkScreen *screen, 
+             const char *geometry, 
+             gboolean browser_window)
 {
     CajaApplication *self = CAJA_APPLICATION (application);
     CajaWindow *window;
@@ -239,9 +240,7 @@ open_window (CajaApplication *application,
     if (browser_window ||g_settings_get_boolean (caja_preferences, CAJA_PREFERENCES_ALWAYS_USE_BROWSER)) {
         window = caja_application_create_navigation_window (application,
                  screen);
-    }
-
-    else {
+    } else {
         window = caja_application_get_spatial_window (application,
                  NULL,
                  NULL,
@@ -254,8 +253,7 @@ open_window (CajaApplication *application,
 
     if (geometry != NULL && !gtk_widget_get_visible (GTK_WIDGET (window))) {
         /* never maximize windows opened from shell if a
-         * custom geometry has been requested.
-         */
+         * custom geometry has been requested. */
         gtk_window_unmaximize (GTK_WINDOW (window));
         eel_gtk_window_set_initial_geometry_from_string (GTK_WINDOW (window),
                                  geometry,
@@ -267,71 +265,142 @@ open_window (CajaApplication *application,
     g_free (uri);
 }
 
+static void
+open_tabs (CajaApplication *application,
+           GFile **locations, 
+           guint n_files, 
+           GdkScreen *screen, 
+           const char *geometry, 
+           gboolean browser_window)
+{
+    CajaApplication *self = CAJA_APPLICATION (application);
+    CajaWindow *window;
+    gchar *uri = NULL;
+
+    /* monitor the preference to use browser or spatial windows */
+    /* connect before trying to read or this preference won't be read by root or after change */
+    g_signal_connect_swapped (caja_preferences, 
+                              "changed::"CAJA_PREFERENCES_ALWAYS_USE_BROWSER,
+                              G_CALLBACK (caja_empty_callback_to_ensure_read),
+                              self);
+
+    if (browser_window ||g_settings_get_boolean (caja_preferences, CAJA_PREFERENCES_ALWAYS_USE_BROWSER)) {
+        window = caja_application_create_navigation_window (application, screen);
+    } else {
+        window = caja_application_get_spatial_window (application,
+                                                      NULL,
+                                                      NULL,
+                                                      locations[0],
+                                                      screen,
+                                                      NULL);
+    }
+
+    /* open all locations */
+    uri = g_file_get_uri (locations[0]);
+    g_debug ("Opening new tab at uri %s\n", uri);
+    caja_window_go_to (window, locations[0]);
+    g_free (uri);
+    for (int i = 1; i< n_files;i++) {
+        /* open tabs in reverse order because each 
+         * tab is opened before the previous one */
+        guint tab = n_files-i;
+        uri = g_file_get_uri (locations[tab]);
+        g_debug ("Opening new tab at uri %s\n", uri);
+        caja_window_go_to_tab (window, locations[tab]);
+        g_free (uri);
+    }
+
+    if (geometry != NULL && !gtk_widget_get_visible (GTK_WIDGET (window))) {
+        /* never maximize windows opened from shell if a
+         * custom geometry has been requested. */
+        gtk_window_unmaximize (GTK_WINDOW (window));
+        eel_gtk_window_set_initial_geometry_from_string (GTK_WINDOW (window),
+                                                         geometry,
+                                                         APPLICATION_WINDOW_MIN_WIDTH,
+                                                         APPLICATION_WINDOW_MIN_HEIGHT,
+                                                         FALSE);
+    }
+}
+
 static void
 open_windows (CajaApplication *application,
-          GFile **files,
-          GdkScreen *screen,
-          const char *geometry,
-          guint len,
-          gboolean browser_window)
+              GFile **files,
+              GdkScreen *screen,
+              const char *geometry,
+              guint n_files,
+              gboolean browser_window,
+              gboolean open_in_tabs)
 {
     guint i;
 
     if (files == NULL || files[0] == NULL) {
         /* Open a window pointing at the default location. */
-        open_window (application, NULL, screen, geometry, browser_window );
+        open_window (application, NULL, screen, geometry, browser_window);
     } else {
-        /* Open windows at each requested location. */ 
-        i = 0;
-        while (i < len ){ 
-              open_window (application, files[i], screen, geometry, browser_window);
-               i++ ;
-         }
+        if (open_in_tabs) {
+            /* Open one window with one tab at each requested location */
+            open_tabs (application, files, n_files, screen, geometry, browser_window);
+        } else {
+            /* Open windows at each requested location. */ 
+            i = 0;
+            while (i < n_files) { 
+                open_window (application, files[i], screen, geometry, browser_window);
+                i++ ;
+            }
+        }
     }
 }
 
 static void
 caja_application_open (GApplication *app,
-               GFile **files,
-               gint n_files,
-               const gchar *options)
+                       GFile **files,
+                       gint n_files,
+                       const gchar *options)
 {
     CajaApplication *self = CAJA_APPLICATION (app);
     gboolean browser_window = FALSE;
+    gboolean open_in_tabs = FALSE;
     const gchar *geometry = NULL;
     const char splitter = '=';
 
     g_debug ("Open called on the GApplication instance; %d files", n_files);
 
-    /*Check if local command line passed --browser or --geometry */
-    if (strcmp(options,"") != 0 ){
-        if (g_str_match_string ("browser",
-                    options,
-                    FALSE) == TRUE){
-            browser_window = TRUE;
-            geometry = strchr(options, splitter);
+    /* Check if local command line passed --browser, --geometry or --tabs */
+    if (strlen (options) > 0) {
+        gchar** splitedOptions = g_strsplit (options, &splitter, 3);
+        sscanf (splitedOptions[0], "%d", &browser_window);
+        if (strcmp (splitedOptions[1], "NULL") != 0) {
+            geometry = splitedOptions[1];
         }
-        else {
-        geometry = options;
-        }
-        /*Reset this or 3ed and later invocations will use same
-	     *geometry even if the user has resized open window
-         */
-        self->priv->geometry = NULL;
-    }
+        sscanf (splitedOptions[2], "%d", &open_in_tabs);
 
-    open_windows (self, files,
-              gdk_screen_get_default (),
-              geometry,
-              n_files,
-              browser_window);
+        open_windows (self, files,
+                      gdk_screen_get_default (),
+                      geometry,
+                      n_files,
+                      browser_window,
+                      open_in_tabs);
+
+        /* Reset this or 3ed and later invocations will use same
+         * geometry even if the user has resized open window */
+        self->priv->geometry = NULL;
+        g_strfreev (splitedOptions);
+    }
+    else
+        open_windows (self, files,
+                      gdk_screen_get_default (),
+                      geometry,
+                      n_files,
+                      browser_window,
+                      open_in_tabs);
 }
 
 void
 caja_application_open_location (CajaApplication *application,
                                 GFile *location,
                                 GFile *selection,
-                                const char *startup_id)
+                                const char *startup_id,
+                                const gboolean open_in_tabs)
 {
     CajaWindow *window;
     GList *sel_list = NULL;
@@ -343,7 +412,8 @@ caja_application_open_location (CajaApplication *application,
     }
 
     caja_window_slot_open_location_full (caja_window_get_active_slot (window), location,
-                                         0, CAJA_WINDOW_OPEN_FLAG_NEW_WINDOW, sel_list, NULL, NULL);
+                                         open_in_tabs, CAJA_WINDOW_OPEN_FLAG_NEW_WINDOW, 
+                                         sel_list, NULL, NULL);
 
     if (sel_list != NULL) {
         caja_file_list_free (sel_list);
@@ -358,7 +428,7 @@ caja_application_quit (CajaApplication *self)
 
     windows = gtk_application_get_windows (GTK_APPLICATION (app));
     g_list_foreach (windows, (GFunc) gtk_widget_destroy, NULL);
-     /* we have been asked to force quit */
+    /* we have been asked to force quit */
     g_application_quit (G_APPLICATION (self));
 }
 
@@ -366,9 +436,7 @@ static void
 caja_application_init (CajaApplication *application)
 {
     GSimpleAction *action;
-    application->priv =
-        G_TYPE_INSTANCE_GET_PRIVATE (application, CAJA_TYPE_APPLICATION,
-                         CajaApplicationPriv);
+    application->priv = caja_application_get_instance_private (application);
     action = g_simple_action_new ("quit", NULL);
 
     g_action_map_add_action (G_ACTION_MAP (application), G_ACTION (action));
@@ -1679,10 +1747,10 @@ caja_application_load_session (CajaApplication *application)
     GKeyFile *state_file;
     char *data;
     caja_application_smclient_initialize (application);
-   if (!egg_sm_client_is_resumed (application->smclient))
+    if (!egg_sm_client_is_resumed (application->smclient))
     {
         return;
-  } 
+    }
 
     state_file = egg_sm_client_get_state_file (application->smclient);
     if (!state_file)
@@ -1979,8 +2047,8 @@ static gboolean
 running_in_mate (void)
 {
     return (g_strcmp0 (g_getenv ("XDG_CURRENT_DESKTOP"), "MATE") == 0)
-        || (g_strcmp0 (g_getenv ("XDG_SESSION_DESKTOP"), "MATE") == 0)
-        || (g_strcmp0 (g_getenv ("DESKTOP_SESSION"), "MATE") == 0);
+        || (g_strcmp0 (g_getenv ("XDG_SESSION_DESKTOP"), "mate") == 0)
+        || (g_strcmp0 (g_getenv ("DESKTOP_SESSION"), "mate") == 0);
 }
 
 static gboolean
@@ -1997,6 +2065,7 @@ caja_application_local_command_line (GApplication *application,
     gboolean perform_self_check = FALSE;
     gboolean version = FALSE;
     gboolean browser_window = FALSE;
+    gboolean open_in_tabs = FALSE;
     gboolean kill_shell = FALSE;
     const gchar *autostart_id;
     gboolean no_default_window = FALSE;
@@ -2022,6 +2091,8 @@ caja_application_local_command_line (GApplication *application,
           N_("Do not manage the desktop (ignore the preference set in the preferences dialog)."), NULL },
         { "force-desktop", '\0', 0, G_OPTION_ARG_NONE, &self->priv->force_desktop,
           N_("Manage the desktop regardless of set preferences or environment (on new startup only)"), NULL },
+        { "tabs", 't', 0, G_OPTION_ARG_NONE, &open_in_tabs, 
+          N_("Open URIs in tabs."), NULL },
         { "browser", '\0', 0, G_OPTION_ARG_NONE, &browser_window, 
           N_("Open a browser window."), NULL },
         { "quit", 'q', 0, G_OPTION_ARG_NONE, &kill_shell, 
@@ -2051,7 +2122,7 @@ caja_application_local_command_line (GApplication *application,
     if (autostart_id != NULL && *autostart_id != '\0') {
         no_default_window = TRUE;
         self->priv->autostart = TRUE;
-        }
+    }
 
 
     argv = *arguments;
@@ -2105,11 +2176,10 @@ caja_application_local_command_line (GApplication *application,
     /* Initialize  and load session info if available */
     /* Load session if and only if autostarted        */
     /* This avoids errors on command line invocation  */
-    if (autostart_id != NULL ) {
+    if (autostart_id != NULL && self->smclient != NULL ) {
         caja_application_load_session (self);
     }
 
-
     GFile **files;
     gint idx, len;
 
@@ -2143,44 +2213,28 @@ caja_application_local_command_line (GApplication *application,
         files[1] = NULL;
     }
 
-    /*Set up geometry and --browser options  */
+    /*Set up --geometry, --browser and --tabs options  */
     /*Invoke "Open" to create new windows */
-
-    if (browser_window == TRUE && self->priv->geometry == NULL){
-
-        if (len > 0)  {
-        g_application_open (application, files, len, "browser");
+    if (len > 0)  {
+        gchar* concatOptions = g_malloc0(64);
+        if (self->priv->geometry == NULL) {
+            g_snprintf (concatOptions, 64, "%d=NULL=%d", browser_window, open_in_tabs);
+        } else {
+            g_snprintf (concatOptions, 64, "%d=%s=%d", browser_window, self->priv->geometry, open_in_tabs);
         }
+        g_application_open (application, files, len, concatOptions);
+        g_free (concatOptions);
     }
 
-    else if (browser_window == FALSE && self->priv->geometry != NULL){
-        if (len > 0)  {
-            g_application_open (application, files, len, self->priv->geometry);
+    if (files)
+        for (idx = 0; idx < len; idx++) {
+            g_object_unref (files[idx]);
         }
-    }
-
-    else if (browser_window == TRUE && self->priv->geometry != NULL){
-        if (len > 0)  {
-            g_application_open (application, files, len, (g_strconcat("browser","=",
-                                self->priv->geometry, NULL)));
-        }
-    }
-
-    else {
-        if (len > 0)  {
-            g_application_open (application, files, len, "");
-        }
-    }
-
-    for (idx = 0; idx < len; idx++) {
-        g_object_unref (files[idx]);
-    }
     g_free (files);
 
  out:
     g_option_context_free (context);
 
-
     return TRUE;    
 }
 
@@ -2420,11 +2474,11 @@ caja_application_quit_mainloop (GApplication *app)
 static void
 caja_application_class_init (CajaApplicationClass *class)
 {
-        GObjectClass *object_class;
+    GObjectClass *object_class;
     GApplicationClass *application_class;
 
-        object_class = G_OBJECT_CLASS (class);
-        object_class->finalize = caja_application_finalize;
+    object_class = G_OBJECT_CLASS (class);
+    object_class->finalize = caja_application_finalize;
 
     application_class = G_APPLICATION_CLASS (class);
     application_class->startup = caja_application_startup;
@@ -2432,7 +2486,6 @@ caja_application_class_init (CajaApplicationClass *class)
     application_class->open = caja_application_open;
     application_class->local_command_line = caja_application_local_command_line;
 
-    g_type_class_add_private (class, sizeof (CajaApplicationPriv));
 }
 
 CajaApplication *
