@@ -37,6 +37,7 @@
 #include <eel/eel-graphic-effects.h>
 #include <eel/eel-gdk-pixbuf-extensions.h>
 #include <eel/eel-stock-dialogs.h>
+#include <eel/eel-gtk-extensions.h>
 #include <gdk-pixbuf/gdk-pixbuf.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
@@ -186,10 +187,8 @@ caja_emblem_sidebar_button_press_cb (GtkWidget *widget,
         gtk_widget_set_sensitive (emblem_sidebar->details->popup_rename,
                                   caja_emblem_can_rename_emblem (keyword));
 
-
-        gtk_menu_popup (GTK_MENU (emblem_sidebar->details->popup),
-                        NULL, NULL, NULL, NULL, event->button,
-                        event->time);
+        gtk_menu_popup_at_pointer (GTK_MENU (emblem_sidebar->details->popup),
+                                             (const GdkEvent*) event);
     }
 
     return TRUE;
@@ -281,16 +280,24 @@ create_rename_emblem_dialog (CajaEmblemSidebar *emblem_sidebar,
     image = gtk_image_new_from_pixbuf (pixbuf);
     entry = gtk_entry_new ();
 
-    dialog = gtk_dialog_new_with_buttons (_("Rename Emblem"),
-                                          NULL,
-                                          0,
-                                          GTK_STOCK_CANCEL,
-                                          GTK_RESPONSE_CANCEL,
-                                          GTK_STOCK_OK,
-                                          GTK_RESPONSE_OK,
-                                          GTK_STOCK_HELP,
-                                          GTK_RESPONSE_HELP,
-                                          NULL);
+    dialog = gtk_dialog_new ();
+    gtk_window_set_title (GTK_WINDOW (dialog), _("Rename Emblem"));
+    gtk_window_set_transient_for (GTK_WINDOW (dialog), NULL);
+
+    eel_dialog_add_button (GTK_DIALOG (dialog),
+                           _("_Cancel"),
+                           "process-stop",
+                           GTK_RESPONSE_CANCEL);
+
+    eel_dialog_add_button (GTK_DIALOG (dialog),
+                           _("_OK"),
+                           "gtk-ok",
+                           GTK_RESPONSE_OK);
+
+    eel_dialog_add_button (GTK_DIALOG (dialog),
+                           _("_Help"),
+                           "help-browser",
+                           GTK_RESPONSE_HELP);
 
     gtk_dialog_set_default_response (GTK_DIALOG (dialog),
                                      GTK_RESPONSE_OK);
@@ -346,17 +353,14 @@ caja_emblem_sidebar_rename_cb (GtkWidget *menu_item,
 static void
 create_popup_menu (CajaEmblemSidebar *emblem_sidebar)
 {
-    GtkWidget *popup, *menu_item, *menu_image;
+    GtkWidget *popup, *menu_item;
 
     popup = gtk_menu_new ();
 
+    gtk_menu_set_reserve_toggle_size (GTK_MENU (popup), FALSE);
+
     /* add the "rename" menu item */
-    menu_image = gtk_image_new_from_stock (GTK_STOCK_PROPERTIES,
-                                           GTK_ICON_SIZE_MENU);
-    gtk_widget_show (menu_image);
-    menu_item = gtk_image_menu_item_new_with_label (_("Rename"));
-    gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (menu_item),
-                                   menu_image);
+    menu_item = eel_image_menu_item_new_from_icon ("document-properties", _("Rename"));
 
     g_signal_connect (menu_item, "activate",
                       G_CALLBACK (caja_emblem_sidebar_rename_cb),
@@ -366,8 +370,8 @@ create_popup_menu (CajaEmblemSidebar *emblem_sidebar)
     emblem_sidebar->details->popup_rename = menu_item;
 
     /* add "delete" menu item */
-    menu_item = gtk_image_menu_item_new_from_stock (GTK_STOCK_DELETE,
-                NULL);
+    menu_item = eel_image_menu_item_new_from_icon ("edit-delete", _("_Delete"));
+
     g_signal_connect (menu_item, "activate",
                       G_CALLBACK (caja_emblem_sidebar_delete_cb),
                       emblem_sidebar);
@@ -533,16 +537,24 @@ create_add_emblems_dialog (CajaEmblemSidebar *emblem_sidebar,
 
     first_entry = NULL;
 
-    dialog = gtk_dialog_new_with_buttons (_("Add Emblems..."),
-                                          NULL,
-                                          0,
-                                          GTK_STOCK_CANCEL,
-                                          GTK_RESPONSE_CANCEL,
-                                          GTK_STOCK_OK,
-                                          GTK_RESPONSE_OK,
-                                          GTK_STOCK_HELP,
-                                          GTK_RESPONSE_HELP,
-                                          NULL);
+    dialog = gtk_dialog_new ();
+    gtk_window_set_title (GTK_WINDOW (dialog), _("Add Emblems..."));
+    gtk_window_set_transient_for (GTK_WINDOW (dialog), NULL);
+
+    eel_dialog_add_button (GTK_DIALOG (dialog),
+                           _("_Cancel"),
+                           "process-stop",
+                           GTK_RESPONSE_CANCEL);
+
+    eel_dialog_add_button (GTK_DIALOG (dialog),
+                           _("_OK"),
+                           "gtk-ok",
+                           GTK_RESPONSE_OK);
+
+    eel_dialog_add_button (GTK_DIALOG (dialog),
+                           _("_Help"),
+                           "help-browser",
+                           GTK_RESPONSE_HELP);
 
     gtk_dialog_set_default_response (GTK_DIALOG (dialog),
                                      GTK_RESPONSE_OK);
@@ -844,7 +856,6 @@ caja_emblem_sidebar_drag_received_cb (GtkWidget *widget,
             if (uri_utf8)
             {
                 error = g_strdup_printf (_("The file '%s' does not appear to be a valid image."), uri_utf8);
-                g_free (uri_utf8);
             }
             else
             {
