@@ -32,6 +32,7 @@
 #include <gio/gio.h>
 
 #include <glib/gi18n.h>
+#include <cairo-gobject.h>
 
 #include <eel/eel-glib-extensions.h>
 
@@ -43,6 +44,8 @@
 
 #include <libcaja-private/caja-autorun.h>
 
+#include <libcaja-extension/caja-configurable.h>
+
 /* string enum preferences */
 #define CAJA_FILE_MANAGEMENT_PROPERTIES_DEFAULT_VIEW_WIDGET "default_view_combobox"
 #define CAJA_FILE_MANAGEMENT_PROPERTIES_ICON_VIEW_ZOOM_WIDGET "icon_view_zoom_combobox"
@@ -66,6 +69,7 @@
 #define CAJA_FILE_MANAGEMENT_PROPERTIES_TRASH_CONFIRM_TRASH_WIDGET "trash_confirm_trash_checkbutton"
 #define CAJA_FILE_MANAGEMENT_PROPERTIES_TRASH_DELETE_WIDGET "trash_delete_checkbutton"
 #define CAJA_FILE_MANAGEMENT_PROPERTIES_SHOW_HIDDEN_WIDGET "hidden_files_checkbutton"
+#define CAJA_FILE_MANAGEMENT_PROPERTIES_SHOW_BACKUP_WIDGET "backup_files_checkbutton"
 #define CAJA_FILE_MANAGEMENT_PROPERTIES_TREE_VIEW_FOLDERS_WIDGET "treeview_folders_checkbutton"
 #define CAJA_FILE_MANAGEMENT_PROPERTIES_MEDIA_AUTOMOUNT_OPEN "media_automount_open_checkbutton"
 #define CAJA_FILE_MANAGEMENT_PROPERTIES_MEDIA_AUTORUN_NEVER "media_autorun_never_checkbutton"
@@ -186,9 +190,9 @@ static const char * const icon_captions_components[] =
 enum
 {
 	EXT_STATE_COLUMN,
-    EXT_ICON_COLUMN,
+	EXT_ICON_COLUMN,
 	EXT_INFO_COLUMN,
-    EXT_STRUCT_COLUMN
+	EXT_STRUCT_COLUMN
 };
 
 static void caja_file_management_properties_dialog_update_media_sensitivity (GtkBuilder *builder);
@@ -622,6 +626,7 @@ out:
 }
 
 static gulong extension_about_id = 0;
+static gulong extension_configure_id = 0;
 
 static void
 extension_about_clicked (GtkButton *button, Extension *ext)
@@ -642,8 +647,29 @@ extension_about_clicked (GtkButton *button, Extension *ext)
     gtk_widget_destroy (GTK_WIDGET (extension_about_dialog));
 }
 
+static int extension_configure_check (Extension *ext)
+{
+    if (!ext->state) // For now, only allow configuring enabled extensions.
+    {
+        return 0;
+    }
+    if (!CAJA_IS_CONFIGURABLE(ext->module))
+    {
+        return 0;
+    }
+    return 1;
+}
+
 static void
-extension_list_selection_changed (GtkTreeSelection *selection, GtkButton *about_button)
+extension_configure_clicked (GtkButton *button, Extension *ext)
+{
+    if (extension_configure_check(ext)) {
+        caja_configurable_run_config(CAJA_CONFIGURABLE(ext->module));
+    }
+}
+
+static void
+extension_list_selection_changed_about (GtkTreeSelection *selection, GtkButton *about_button)
 {
     GtkTreeModel *model;
     GtkTreeIter iter;
@@ -667,6 +693,34 @@ extension_list_selection_changed (GtkTreeSelection *selection, GtkButton *about_
     }
 }
 
+static void
+extension_list_selection_changed_configure (GtkTreeSelection *selection, GtkButton *configure_button)
+{
+    GtkTreeModel *model;
+    GtkTreeIter iter;
+    Extension *ext;
+
+    gtk_widget_set_sensitive (GTK_WIDGET (configure_button), FALSE);
+
+    if (extension_configure_id > 0)
+    {
+        g_signal_handler_disconnect (configure_button, extension_configure_id);
+        extension_configure_id = 0;
+    }
+
+    if (!gtk_tree_selection_get_selected (selection, &model, &iter))
+        return;
+
+    gtk_tree_model_get (model, &iter, EXT_STRUCT_COLUMN, &ext, -1);
+    if (ext != NULL) {
+        // Unconfigurable extensions remain unconfigurable.
+        if (extension_configure_check(ext)) {
+            gtk_widget_set_sensitive (GTK_WIDGET (configure_button), TRUE);
+            extension_configure_id = g_signal_connect (configure_button, "clicked", G_CALLBACK (extension_configure_clicked), ext);
+        }
+    }
+}
+
 static void
 extension_state_toggled (GtkCellRendererToggle *cell, gchar *path_str, gpointer data)
 {
@@ -725,7 +779,7 @@ caja_file_management_properties_dialog_setup_media_page (GtkBuilder *builder)
     other_type_combo_box = GTK_WIDGET (gtk_builder_get_object (builder, "media_other_type_combobox"));
 
     other_type_list_store = gtk_list_store_new (3,
-                            GDK_TYPE_PIXBUF,
+                            CAIRO_GOBJECT_TYPE_SURFACE,
                             G_TYPE_STRING,
                             G_TYPE_STRING);
 
@@ -741,7 +795,7 @@ caja_file_management_properties_dialog_setup_media_page (GtkBuilder *builder)
         char *description;
         GIcon *icon;
         CajaIconInfo *icon_info;
-        GdkPixbuf *pixbuf;
+        cairo_surface_t *surface;
         int icon_size, icon_scale;
 
         if (!g_str_has_prefix (content_type, "x-content/"))
@@ -764,21 +818,21 @@ caja_file_management_properties_dialog_setup_media_page (GtkBuilder *builder)
         {
             icon_info = caja_icon_info_lookup (icon, icon_size, icon_scale);
             g_object_unref (icon);
-            pixbuf = caja_icon_info_get_pixbuf_nodefault_at_size (icon_info, icon_size);
+            surface = caja_icon_info_get_surface_nodefault_at_size (icon_info, icon_size);
             g_object_unref (icon_info);
         }
         else
         {
-            pixbuf = NULL;
+            surface = NULL;
         }
 
         gtk_list_store_set (other_type_list_store, &iter,
-                            0, pixbuf,
+                            0, surface,
                             1, description,
                             2, content_type,
                             -1);
-        if (pixbuf != NULL)
-            g_object_unref (pixbuf);
+        if (surface != NULL)
+            cairo_surface_destroy (surface);
         g_free (description);
 skip:
         ;
@@ -791,7 +845,7 @@ skip:
     renderer = gtk_cell_renderer_pixbuf_new ();
     gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (other_type_combo_box), renderer, FALSE);
     gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (other_type_combo_box), renderer,
-                                    "pixbuf", 0,
+                                    "surface", 0,
                                     NULL);
     renderer = gtk_cell_renderer_text_new ();
     gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (other_type_combo_box), renderer, TRUE);
@@ -818,8 +872,8 @@ caja_file_management_properties_dialog_setup_extension_page (GtkBuilder *builder
     GtkTreeSelection *selection;
     GtkTreeIter iter;
     GtkIconTheme *icon_theme;
-    GdkPixbuf *ext_pixbuf_icon;
-    GtkButton *about_button;
+    cairo_surface_t *ext_surface_icon;
+    GtkButton *about_button, *configure_button;
     gchar *ext_text_info;
     
     GList *extensions;
@@ -847,15 +901,15 @@ caja_file_management_properties_dialog_setup_extension_page (GtkBuilder *builder
 
         if (ext->icon != NULL)
         {
-            ext_pixbuf_icon = gtk_icon_theme_load_icon (icon_theme, ext->icon,
-                                                        24,
-                                                        GTK_ICON_LOOKUP_USE_BUILTIN, NULL);
+            ext_surface_icon = gtk_icon_theme_load_surface (icon_theme, ext->icon,
+                                                            24, gtk_widget_get_scale_factor (GTK_WIDGET (view)),
+                                                            NULL, GTK_ICON_LOOKUP_USE_BUILTIN, NULL);
         }
         else
         {
-            ext_pixbuf_icon = gtk_icon_theme_load_icon (icon_theme, "system-run",
-                                                        24,
-                                                        GTK_ICON_LOOKUP_USE_BUILTIN, NULL);
+            ext_surface_icon = gtk_icon_theme_load_surface (icon_theme, "system-run",
+                                                            24, gtk_widget_get_scale_factor (GTK_WIDGET (view)),
+                                                            NULL, GTK_ICON_LOOKUP_USE_BUILTIN, NULL);
         }
 
         if (ext->description != NULL)
@@ -873,21 +927,26 @@ caja_file_management_properties_dialog_setup_extension_page (GtkBuilder *builder
         gtk_list_store_append (store, &iter);
         gtk_list_store_set (store, &iter,
                             EXT_STATE_COLUMN, ext->state,
-                            EXT_ICON_COLUMN, ext_pixbuf_icon, 
+                            EXT_ICON_COLUMN, ext_surface_icon,
                             EXT_INFO_COLUMN, ext_text_info,
                             EXT_STRUCT_COLUMN, ext, -1);
 
         g_free (ext_text_info);
-        if (ext_pixbuf_icon)
-            g_object_unref (ext_pixbuf_icon);
+        if (ext_surface_icon)
+            cairo_surface_destroy (ext_surface_icon);
     }
 
     about_button = GTK_BUTTON (gtk_builder_get_object (builder, "about_extension_button"));
+    configure_button = GTK_BUTTON (gtk_builder_get_object (builder, "configure_extension_button"));
+
     selection = gtk_tree_view_get_selection (view);
     gtk_tree_selection_set_mode (selection, GTK_SELECTION_SINGLE);
     g_signal_connect (selection, "changed",
-                      G_CALLBACK (extension_list_selection_changed),
+                      G_CALLBACK (extension_list_selection_changed_about),
                       about_button);
+    g_signal_connect (selection, "changed",
+                      G_CALLBACK (extension_list_selection_changed_configure),
+                      configure_button);
 }
 
 static void
@@ -1121,6 +1180,9 @@ caja_file_management_properties_dialog_setup (GtkBuilder *builder, GtkWindow *wi
     bind_builder_bool (builder, caja_preferences,
                        CAJA_FILE_MANAGEMENT_PROPERTIES_SHOW_HIDDEN_WIDGET,
                        CAJA_PREFERENCES_SHOW_HIDDEN_FILES);
+    bind_builder_bool (builder, caja_preferences,
+                       CAJA_FILE_MANAGEMENT_PROPERTIES_SHOW_BACKUP_WIDGET,
+                       CAJA_PREFERENCES_SHOW_BACKUP_FILES);
     bind_builder_bool (builder, caja_tree_sidebar_preferences,
                        CAJA_FILE_MANAGEMENT_PROPERTIES_TREE_VIEW_FOLDERS_WIDGET,
                        CAJA_PREFERENCES_TREE_SHOW_ONLY_DIRECTORIES);
