@@ -59,7 +59,7 @@ static const char untranslated_go_to_label[] = N_("Go To:");
 #define LOCATION_LABEL _(untranslated_location_label)
 #define GO_TO_LABEL _(untranslated_go_to_label)
 
-struct CajaLocationBarDetails
+struct _CajaLocationBarPrivate
 {
     GtkLabel *label;
     CajaEntry *entry;
@@ -97,7 +97,7 @@ static const GtkTargetEntry drop_types [] =
     { CAJA_DND_TEXT_PLAIN_TYPE, 0, CAJA_DND_TEXT_PLAIN },
 };
 
-G_DEFINE_TYPE (CajaLocationBar, caja_location_bar, GTK_TYPE_BOX);
+G_DEFINE_TYPE_WITH_PRIVATE (CajaLocationBar, caja_location_bar, GTK_TYPE_BOX);
 
 static CajaNavigationWindow *
 caja_location_bar_get_window (GtkWidget *bar)
@@ -468,8 +468,6 @@ caja_location_bar_class_init (CajaLocationBarClass *klass)
 
     binding_set = gtk_binding_set_by_class (klass);
     gtk_binding_entry_add_signal (binding_set, GDK_KEY_Escape, 0, "cancel", 0);
-
-    g_type_class_add_private (klass, sizeof (CajaLocationBarDetails));
 }
 
 static void
@@ -479,8 +477,7 @@ caja_location_bar_init (CajaLocationBar *bar)
     GtkWidget *entry;
     GtkWidget *event_box;
 
-    bar->details = G_TYPE_INSTANCE_GET_PRIVATE (bar, CAJA_TYPE_LOCATION_BAR,
-                                                CajaLocationBarDetails);
+    bar->details = caja_location_bar_get_instance_private (bar);
 
     gtk_orientable_set_orientation (GTK_ORIENTABLE (bar),
                                     GTK_ORIENTATION_HORIZONTAL);
@@ -592,30 +589,63 @@ caja_location_bar_set_location (CajaLocationBar *bar,
     caja_location_bar_update_label (bar);
 }
 
+static void
+override_background_color (GtkWidget *widget,
+                           GdkRGBA   *rgba)
+{
+    gchar          *css;
+    GtkCssProvider *provider;
+
+    provider = gtk_css_provider_new ();
+
+    css = g_strdup_printf ("entry { background-color: %s;}",
+                           gdk_rgba_to_string (rgba));
+    gtk_css_provider_load_from_data (provider, css, -1, NULL);
+    g_free (css);
+
+    gtk_style_context_add_provider (gtk_widget_get_style_context (widget),
+                                    GTK_STYLE_PROVIDER (provider),
+                                    GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+    g_object_unref (provider);
+}
+
 /* change background color based on activity state */
 void
-caja_location_bar_set_active(CajaLocationBar *location_bar, gboolean is_active)
+caja_location_bar_set_active (CajaLocationBar *location_bar, gboolean is_active)
 {
+    GtkStyleContext *style;
+    GdkRGBA color;
+    GdkRGBA *c;
+    static GdkRGBA bg_active;
+    static GdkRGBA bg_inactive;
+
+    style = gtk_widget_get_style_context (GTK_WIDGET (location_bar->details->entry));
+
     if (is_active)
-    {
-        /* reset style to default */
-        gtk_widget_override_background_color (GTK_WIDGET (location_bar->details->entry), GTK_STATE_FLAG_NORMAL, NULL);
-    }
+        gtk_style_context_get (style, GTK_STATE_FLAG_NORMAL,
+                               GTK_STYLE_PROPERTY_BACKGROUND_COLOR,
+                               &c, NULL);
     else
-    {
-        GtkStyleContext *style;
-        GdkRGBA color;
-        GdkRGBA *c;
-
-        style = gtk_widget_get_style_context (GTK_WIDGET (location_bar->details->entry));
-
         gtk_style_context_get (style, GTK_STATE_FLAG_INSENSITIVE,
                                GTK_STYLE_PROPERTY_BACKGROUND_COLOR,
                                &c, NULL);
-        color = *c;
-        gdk_rgba_free (c);
 
-        gtk_widget_override_background_color (GTK_WIDGET (location_bar->details->entry), GTK_STATE_FLAG_ACTIVE, &color);
+    color = *c;
+    gdk_rgba_free (c);
+
+    if (is_active)
+    {
+        if (gdk_rgba_equal (&bg_active, &bg_inactive))
+            bg_active = color;
+
+        override_background_color (GTK_WIDGET (location_bar->details->entry), &bg_active);
+    }
+    else
+    {
+        if (gdk_rgba_equal (&bg_active, &bg_inactive))
+            bg_inactive = color;
+
+        override_background_color(GTK_WIDGET (location_bar->details->entry), &bg_inactive);
     }
 }
 
