@@ -39,11 +39,7 @@
 static void caja_navigation_action_init       (CajaNavigationAction *action);
 static void caja_navigation_action_class_init (CajaNavigationActionClass *class);
 
-static GObjectClass *parent_class = NULL;
-
-#define CAJA_NAVIGATION_ACTION_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), CAJA_TYPE_NAVIGATION_ACTION, CajaNavigationActionPrivate))
-
-struct CajaNavigationActionPrivate
+struct _CajaNavigationActionPrivate
 {
     CajaNavigationWindow *window;
     CajaNavigationDirection direction;
@@ -58,33 +54,7 @@ enum
     PROP_WINDOW
 };
 
-GType
-caja_navigation_action_get_type (void)
-{
-    static GType type = 0;
-
-    if (type == 0)
-    {
-        const GTypeInfo type_info =
-        {
-            sizeof (CajaNavigationActionClass),
-            (GBaseInitFunc) NULL,
-            (GBaseFinalizeFunc) NULL,
-            (GClassInitFunc) caja_navigation_action_class_init,
-            (GClassFinalizeFunc) NULL,
-            NULL,
-            sizeof (CajaNavigationAction),
-            0, /* n_preallocs */
-            (GInstanceInitFunc) caja_navigation_action_init,
-        };
-
-        type = g_type_register_static (GTK_TYPE_ACTION,
-                                       "CajaNavigationAction",
-                                       &type_info, 0);
-    }
-
-    return type;
-}
+G_DEFINE_TYPE_WITH_PRIVATE (CajaNavigationAction, caja_navigation_action, GTK_TYPE_ACTION)
 
 static gboolean
 should_open_in_new_tab (void)
@@ -139,6 +109,7 @@ fill_menu (CajaNavigationWindow *window,
     GtkWidget *menu_item;
     int index;
     GList *list;
+    gboolean list_void;
 
     g_assert (CAJA_IS_NAVIGATION_WINDOW (window));
 
@@ -146,21 +117,44 @@ fill_menu (CajaNavigationWindow *window,
 
     list = back ? slot->back_list : slot->forward_list;
     index = 0;
+    list_void = TRUE;
+
     while (list != NULL)
     {
         menu_item = caja_bookmark_menu_item_new (CAJA_BOOKMARK (list->data));
-        g_object_set_data (G_OBJECT (menu_item), "user_data", GINT_TO_POINTER (index));
-        gtk_widget_show (GTK_WIDGET (menu_item));
-        g_signal_connect_object (menu_item, "activate",
-                                 back
-                                 ? G_CALLBACK (activate_back_menu_item_callback)
-                                 : G_CALLBACK (activate_forward_menu_item_callback),
-                                 window, 0);
 
-        gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+        if (menu_item) {
+            list_void = FALSE;
+            g_object_set_data (G_OBJECT (menu_item), "user_data", GINT_TO_POINTER (index));
+            gtk_widget_show (GTK_WIDGET (menu_item));
+            g_signal_connect_object (menu_item, "activate",
+                                     back
+                                     ? G_CALLBACK (activate_back_menu_item_callback)
+                                     : G_CALLBACK (activate_forward_menu_item_callback),
+                                     window, 0);
+
+            gtk_menu_shell_append (GTK_MENU_SHELL (menu), menu_item);
+        }
+
         list = g_list_next (list);
         ++index;
     }
+
+    if (list_void)
+    {
+        gtk_menu_shell_append (GTK_MENU_SHELL (menu),
+                               eel_image_menu_item_new_from_icon ("dialog-error", _("folder removed")));
+        if (back)
+        {
+            caja_navigation_window_slot_clear_back_list (slot);
+            caja_navigation_window_allow_back (window, FALSE);
+        }
+        else
+        {
+            caja_navigation_window_slot_clear_forward_list (slot);
+            caja_navigation_window_allow_forward (window, FALSE);
+        }
+    }
 }
 
 static void
@@ -255,7 +249,7 @@ connect_proxy (GtkAction *action, GtkWidget *proxy)
         g_signal_connect (child, "button-release-event", G_CALLBACK (proxy_button_release_event_cb), NULL);
     }
 
-    (* GTK_ACTION_CLASS (parent_class)->connect_proxy) (action, proxy);
+    (* GTK_ACTION_CLASS (caja_navigation_action_parent_class)->connect_proxy) (action, proxy);
 }
 
 static void
@@ -272,7 +266,7 @@ disconnect_proxy (GtkAction *action, GtkWidget *proxy)
         g_signal_handlers_disconnect_by_func (child, G_CALLBACK (proxy_button_release_event_cb), NULL);
     }
 
-    (* GTK_ACTION_CLASS (parent_class)->disconnect_proxy) (action, proxy);
+    (* GTK_ACTION_CLASS (caja_navigation_action_parent_class)->disconnect_proxy) (action, proxy);
 }
 
 static void
@@ -282,7 +276,7 @@ caja_navigation_action_finalize (GObject *object)
 
     g_free (action->priv->arrow_tooltip);
 
-    (* G_OBJECT_CLASS (parent_class)->finalize) (object);
+    (* G_OBJECT_CLASS (caja_navigation_action_parent_class)->finalize) (object);
 }
 
 static void
@@ -344,8 +338,6 @@ caja_navigation_action_class_init (CajaNavigationActionClass *class)
     object_class->set_property = caja_navigation_action_set_property;
     object_class->get_property = caja_navigation_action_get_property;
 
-    parent_class = g_type_class_peek_parent (class);
-
     action_class->toolbar_item_type = GTK_TYPE_MENU_TOOL_BUTTON;
     action_class->connect_proxy = connect_proxy;
     action_class->disconnect_proxy = disconnect_proxy;
@@ -373,12 +365,10 @@ caja_navigation_action_class_init (CajaNavigationActionClass *class)
                                              "The navigation window",
                                              G_TYPE_OBJECT,
                                              G_PARAM_READWRITE));
-
-    g_type_class_add_private (object_class, sizeof(CajaNavigationActionPrivate));
 }
 
 static void
 caja_navigation_action_init (CajaNavigationAction *action)
 {
-    action->priv = CAJA_NAVIGATION_ACTION_GET_PRIVATE (action);
+    action->priv = caja_navigation_action_get_instance_private (action);
 }
