@@ -169,13 +169,17 @@ update_button_types (CajaPathBar *path_bar)
         button_data = BUTTON_DATA (list->data);
         if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (button_data->button)))
         {
-            path = button_data->path;
+            /*Increase the reference count on path so it does not get cleared
+             *by caja_path_bar_clear_buttons during caja_path_bar_update_path
+             */
+            path = g_object_ref (button_data->path);
             break;
         }
     }
     if (path != NULL)
     {
         caja_path_bar_update_path (path_bar, path, TRUE);
+        g_object_unref (path);
     }
 }
 
@@ -817,7 +821,6 @@ caja_path_bar_size_allocate (GtkWidget     *widget,
         if (direction == GTK_TEXT_DIR_RTL)
         {
             child_allocation.x -= path_bar->spacing;
-            down_slider_offset = child_allocation.x - widget_allocation.x - path_bar->slider_width;
             down_slider_offset = child_allocation.x - allocation->x - path_bar->slider_width;
         }
         else
@@ -1901,12 +1904,13 @@ make_directory_button (CajaPathBar  *path_bar,
     switch (button_data->type)
     {
     case ROOT_BUTTON:
-        child = button_data->image;
-        button_data->label = NULL;
-        break;
+    /* Fall through */
     case HOME_BUTTON:
+    /* Fall through */
     case DESKTOP_BUTTON:
+    /* Fall through */
     case MOUNT_BUTTON:
+    /* Fall through */
     case DEFAULT_LOCATION_BUTTON:
         button_data->label = gtk_label_new (NULL);
         child = gtk_box_new (GTK_ORIENTATION_HORIZONTAL, 2);
@@ -2057,7 +2061,6 @@ caja_path_bar_update_path (CajaPathBar *path_bar,
     fake_root = NULL;
     result = TRUE;
     first_directory = TRUE;
-    last_directory = FALSE;
     new_buttons = NULL;
     current_button_data = NULL;
 
@@ -2104,6 +2107,7 @@ caja_path_bar_update_path (CajaPathBar *path_bar,
     }
 
     path_bar->current_path = g_object_ref (file_path);
+
     path_bar->current_button_data = current_button_data;
 
     return result;
