@@ -40,7 +40,7 @@
 struct _CajaNavigationWindowPane;
 
 /* FIXME bugzilla.gnome.org 42575: Migrate more fields into here. */
-struct CajaWindowDetails
+struct _CajaWindowPrivate
 {
     GtkWidget *grid;
 
@@ -60,6 +60,7 @@ struct CajaWindowDetails
     CajaBookmarkList *bookmark_list;
 
     CajaWindowShowHiddenFilesMode show_hidden_files_mode;
+    CajaWindowShowBackupFilesMode show_backup_files_mode;
 
     /* View As menu */
     GList *short_list_viewers;
@@ -90,7 +91,7 @@ struct CajaWindowDetails
     gboolean initiated_unmount;
 };
 
-struct _CajaNavigationWindowDetails
+struct _CajaNavigationWindowPrivate
 {
     GtkWidget *content_paned;
     GtkWidget *content_box;
