@@ -100,6 +100,7 @@ static void action_view_as_callback         (GtkAction               *action,
 static GList *history_list;
 
 G_DEFINE_TYPE_WITH_CODE (CajaWindow, caja_window, GTK_TYPE_WINDOW,
+                         G_ADD_PRIVATE (CajaWindow)
                          G_IMPLEMENT_INTERFACE (CAJA_TYPE_WINDOW_INFO,
                                  caja_window_info_iface_init));
 
@@ -152,7 +153,7 @@ caja_window_init (CajaWindow *window)
     }
 
     g_object_unref (provider);
-    window->details = G_TYPE_INSTANCE_GET_PRIVATE (window, CAJA_TYPE_WINDOW, CajaWindowDetails);
+    window->details = caja_window_get_instance_private (window);
 
     window->details->panes = NULL;
     window->details->active_pane = NULL;
@@ -234,6 +235,14 @@ caja_window_go_to (CajaWindow *window, GFile *location)
     caja_window_slot_go_to (window->details->active_pane->active_slot, location, FALSE);
 }
 
+void
+caja_window_go_to_tab (CajaWindow *window, GFile *location)
+{
+    g_return_if_fail (CAJA_IS_WINDOW (window));
+
+    caja_window_slot_go_to (window->details->active_pane->active_slot, location, TRUE);
+}
+
 void
 caja_window_go_to_full (CajaWindow *window,
                         GFile                 *location,
@@ -266,7 +275,7 @@ caja_window_new_tab (CajaWindow *window)
     CajaWindowSlot *current_slot;
     CajaWindowSlot *new_slot;
     CajaWindowOpenFlags flags;
-    GFile *location;
+    GFile *location = NULL;
     int new_slot_position;
     char *scheme;
 
@@ -531,7 +540,8 @@ caja_window_set_initial_window_geometry (CajaWindow *window)
     GdkScreen *screen;
     guint max_width_for_screen, max_height_for_screen;
 
-    guint default_width, default_height;
+    guint default_width = 0;
+    guint default_height = 0;
 
     screen = gtk_window_get_screen (GTK_WINDOW (window));
 
@@ -1967,6 +1977,21 @@ caja_window_set_hidden_files_mode (CajaWindowInfo *window,
     g_signal_emit_by_name (window, "hidden_files_mode_changed");
 }
 
+static CajaWindowShowBackupFilesMode
+caja_window_get_backup_files_mode (CajaWindowInfo *window)
+{
+    return window->details->show_backup_files_mode;
+}
+
+static void
+caja_window_set_backup_files_mode (CajaWindowInfo *window,
+                                   CajaWindowShowBackupFilesMode  mode)
+{
+    window->details->show_backup_files_mode = mode;
+
+    g_signal_emit_by_name (window, "backup_files_mode_changed");
+}
+
 static gboolean
 caja_window_get_initiated_unmount (CajaWindowInfo *window)
 {
@@ -2068,6 +2093,10 @@ caja_window_info_iface_init (CajaWindowInfoIface *iface)
     iface->get_selection = caja_window_get_selection;
     iface->get_hidden_files_mode = caja_window_get_hidden_files_mode;
     iface->set_hidden_files_mode = caja_window_set_hidden_files_mode;
+    
+    iface->get_backup_files_mode = caja_window_get_backup_files_mode;
+    iface->set_backup_files_mode = caja_window_set_backup_files_mode;
+
     iface->get_active_slot = caja_window_get_active_slot;
     iface->get_extra_slot = caja_window_get_extra_slot;
     iface->get_initiated_unmount = caja_window_get_initiated_unmount;
@@ -2159,8 +2188,4 @@ caja_window_class_init (CajaWindowClass *class)
 
     class->reload = caja_window_reload;
     class->go_up = caja_window_go_up_signal;
-
-
-
-    g_type_class_add_private (G_OBJECT_CLASS (class), sizeof (CajaWindowDetails));
 }
